@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 # Main Madpack installation executable.
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -23,16 +23,21 @@ from utilities import info_
 from utilities import is_rev_gte
 from utilities import remove_comments_from_sql
 from utilities import run_query
-# Required Python version
-py_min_ver = [2, 6]
 
-# Check python version
-if sys.version_info[:2] < py_min_ver:
+# Required Python version
+py_min_ver = [2, 7]
+
+if list(sys.version_info[:2]) < py_min_ver:
     print("ERROR: python version too old ({0}). You need {1} or greater.".
           format('.'.join(map(str, sys.version_info[:3])),
                  '.'.join(map(str, py_min_ver))))
     exit(1)
 
+# raw_input isn't defined in Python3.x, whereas input wasn't behaving like raw_input in Python 2.x
+# this should make both input and raw_input work in Python 2.x/3.x like the raw_input from Python 2.x
+try: input = raw_input
+except NameError: raw_input = input
+
 # Find MADlib root directory. This file is installed to
 # $MADLIB_ROOT/madpack/madpack.py, so to get $MADLIB_ROOT we need to go
 # two levels up in the directory hierarchy. We use (a) os.path.realpath and
@@ -82,7 +87,7 @@ def _make_dir(dir):
         try:
             os.makedirs(dir)
         except:
-            print "ERROR: can not create directory: %s. Check permissions." % dir
+            print("ERROR: can not create directory: %s. Check permissions." % dir)
             exit(1)
 # ------------------------------------------------------------------------------
 
@@ -328,9 +333,9 @@ def _parse_result_logfile(retval, logfile, sql_abspath,
 
     if is_install_check_logfile:
         # Output result
-        print "TEST CASE RESULT|Module: " + module + \
+        print("TEST CASE RESULT|Module: " + module + \
             "|" + os.path.basename(sql_filename) + "|" + result + \
-            "|Time: %d milliseconds" % (milliseconds)
+            "|Time: %d milliseconds" % (milliseconds))
 
     if result == 'FAIL':
         error_(this, "Failed executing %s" % sql_abspath, stop=False)
@@ -388,16 +393,16 @@ def _plpy_check(py_min_ver):
 
     # Check PL/Python existence
     rv = _internal_run_query("SELECT count(*) AS CNT FROM pg_language "
-                             "WHERE lanname = 'plpythonu'", True)
+                             "WHERE lanname = 'plpython3u'", True)
     if int(rv[0]['cnt']) > 0:
         info_(this, "> PL/Python already installed", verbose)
     else:
         info_(this, "> PL/Python not installed", verbose)
         info_(this, "> Creating language PL/Python...", True)
         try:
-            _internal_run_query("CREATE LANGUAGE plpythonu;", True)
+            _internal_run_query("CREATE LANGUAGE plpython3u;", True)
         except:
-            error_(this, """Cannot create language plpythonu. Please check if you
+            error_(this, """Cannot create language plpython3u. Please check if you
                 have configured and installed portid (your platform) with
                 `--with-python` option. Stopping installation...""", False)
             raise Exception
@@ -412,7 +417,7 @@ def _plpy_check(py_min_ver):
             # return '.'.join(str(item) for item in sys.version_info[:3])
             return str(sys.version_info[:3]).replace(',','.').replace(' ','').replace(')','').replace('(','')
         $$
-        LANGUAGE plpythonu;
+        LANGUAGE plpython3u;
     """, True)
     rv = _internal_run_query("SELECT plpy_version_for_madlib() AS ver;", True)
     python = rv[0]['ver']
@@ -753,8 +758,18 @@ def _execute_per_module_unit_test_algo(module, pyfile, cur_tmpdir):
         milliseconds = 0
         run_start = datetime.datetime.now()
         # Run the python unit test file
-        runcmd = ["python", pyfile]
-        runenv = os.environ
+        runcmd = ["python3", pyfile]
+        # runenv = os.environ
+        runenv = os.environ.copy()
+
+        # GPDB6 python3 support is provided by an additional package.
+        # To access it, we will have to set environment variables.
+        if dbver == '6':
+            gphome = runenv["GPHOME"]
+            runenv["LD_LIBRARY_PATH"] = "{0}/ext/python3.9/lib:".format(gphome) + runenv["LD_LIBRARY_PATH"]
+            runenv["PATH"] = "{0}/ext/python3.9/bin:".format(gphome) + runenv["PATH"]
+            runenv["PYTHONHOME"] = "{0}/ext/python3.9".format(gphome)
+            runenv["PYTHONPATH"] = "{0}/ext/python3.9/lib".format(gphome)
         retval = subprocess.call(runcmd, env=runenv, stdout=log, stderr=log)
         run_end = datetime.datetime.now()
         milliseconds = round((run_end - run_start).seconds * 1000 +
@@ -1305,9 +1320,14 @@ def set_dynamic_library_path_in_database(dbver_split, madlib_library_path):
 
     global dynamic_library_path
     dynamic_library_path = _internal_run_query("SHOW dynamic_library_path", True)[0]['dynamic_library_path']
+    # GP7 gpconfig messes up $libdir so we remove it for now
+    paths = dynamic_library_path.split(":")
+    if madlib_library_path not in paths:
+        if '$libdir' in paths:
+            paths.remove('$libdir')
 
-    if madlib_library_path not in dynamic_library_path.split(":"):
-        dynamic_library_path = dynamic_library_path + ':' + madlib_library_path
+        paths.append(madlib_library_path)
+        dynamic_library_path = ':'.join(paths)
 
         if portid == 'greenplum':
             if is_rev_gte(dbver_split, get_rev_num('6.0')):
@@ -1338,7 +1358,7 @@ def main(argv):
     global tmpdir
     try:
         tmpdir = tempfile.mkdtemp('', 'madlib.', args.tmpdir)
-    except OSError, e:
+    except OSError as e:
         tmpdir = e.filename
         error_(this, "cannot create temporary directory: '%s'." % tmpdir, True)
 
@@ -1413,6 +1433,7 @@ def main(argv):
         supportedVersions = [dirItem for dirItem in os.listdir(portdir)
                              if os.path.isdir(os.path.join(portdir, dirItem)) and
                              re.match("^\d+", dirItem)]
+
         if dbver is None:
             dbver = ".".join(
                 map(str, max([versionStr.split('.')
@@ -1541,4 +1562,4 @@ if __name__ == "__main__":
     if not keeplogs:
         shutil.rmtree(tmpdir)
     else:
-        print "INFO: Log files saved in " + tmpdir
+        print("INFO: Log files saved in " + tmpdir)
